@@ -5,19 +5,18 @@ import datetime
 from email import message_from_string
 from email.policy import SMTP as SMTP_POLICY
 from email.headerregistry import Address, AddressHeader, Group
+import locale
 import logging
 import operator
 import os
 import sys
 import textwrap
-from typing import cast, Callable, Dict, Iterable, List, Optional, Union
-
-from unidecode import unidecode
+from typing import cast, Callable, Iterable, Optional, Union
 
 from . import helpers
-from .address_book import (AddressBookCollection, AddressBookNameError,
-                           AddressBookParseError, VdirAddressBook)
-from .carddav_object import CarddavObject
+from .address_book import AddressBookCollection, VdirAddressBook
+from .exceptions import AddressBookNameError, AddressBookParseError, Cancelled
+from .contacts import Contact
 from . import cli
 from .config import Config
 from .formatter import Formatter
@@ -29,9 +28,11 @@ from .version import version as khard_version
 
 logger = logging.getLogger(__name__)
 config: Config
+# the types that sys.exit() can digest
+ExitStatus = Union[str, int, None]
 
 
-def version_check(contact: CarddavObject, description: str) -> bool:
+def version_check(contact: Contact, description: str) -> bool:
     if contact.version not in config.supported_vcard_versions:
         print("Warning:\nThe {} is based on vcard version {} but khard only "
               "supports the modification of vcards with version 3.0 and 4.0.\n"
@@ -52,11 +53,11 @@ def create_new_contact(address_book: VdirAddressBook) -> None:
         "{}\n# if you want to cancel, exit without saving\n\n{}".format(
             address_book, config.preferred_vcard_version,
             helpers.get_new_contact_template(config.private_objects))
-    new_contact = editor.edit_templates(lambda t: CarddavObject.from_yaml(
+    new_contact = editor.edit_templates(lambda t: Contact.from_yaml(
         address_book, t, config.private_objects,
         config.preferred_vcard_version, config.localize_dates), template)
 
-    # create carddav object from temp file
+    # create contact object from temp file
     if new_contact is None:
         print("Canceled")
     else:
@@ -64,7 +65,7 @@ def create_new_contact(address_book: VdirAddressBook) -> None:
         print("Creation successful\n\n{}".format(new_contact.pretty()))
 
 
-def modify_existing_contact(old_contact: CarddavObject) -> None:
+def modify_existing_contact(old_contact: Contact) -> None:
     editor = interactive.Editor(config.editor, config.merge_editor)
     # create temp file and open it with the specified text editor
     text = ("# Edit contact: {}\n# Address book: {}\n# Vcard version: {}\n"
@@ -72,7 +73,7 @@ def modify_existing_contact(old_contact: CarddavObject) -> None:
                 old_contact, old_contact.address_book, old_contact.version,
                 old_contact.to_yaml()))
     new_contact = editor.edit_templates(
-        lambda t: CarddavObject.clone_with_yaml_update(
+        lambda t: Contact.clone_with_yaml_update(
             old_contact, t, config.localize_dates), text)
 
     # check if the user changed anything
@@ -83,8 +84,8 @@ def modify_existing_contact(old_contact: CarddavObject) -> None:
         print("Modification successful\n\n{}".format(new_contact.pretty()))
 
 
-def merge_existing_contacts(source_contact: CarddavObject,
-                            target_contact: CarddavObject,
+def merge_existing_contacts(source_contact: Contact,
+                            target_contact: Contact,
                             delete_source_contact: bool) -> None:
     # show warning, if target vCard version is not 3.0 or 4.0
     if not version_check(target_contact, "target contact in which to merge"):
@@ -100,13 +101,12 @@ def merge_existing_contacts(source_contact: CarddavObject,
                        target_contact, target_contact.address_book,
                        target_contact.version, target_contact.to_yaml()))
     merged_contact = editor.edit_templates(
-        lambda t: CarddavObject.clone_with_yaml_update(
+        lambda t: Contact.clone_with_yaml_update(
             target_contact, t, config.localize_dates), src_text, target_text)
     # compare them
     if merged_contact is None or target_contact == merged_contact:
-        print("Target contact unmodified\n\n{}".format(
-            target_contact.pretty()))
-        sys.exit(0)
+        raise Cancelled(
+            f"Target contact unmodified\n\n{target_contact.pretty()}", 0)
 
     print("Merge contact {} from address book {} into contact {} from address "
           "book {}\n\n".format(source_contact, source_contact.address_book,
@@ -128,7 +128,7 @@ def merge_existing_contacts(source_contact: CarddavObject,
     print("Merge successful\n\n{}".format(merged_contact.pretty()))
 
 
-def copy_contact(contact: CarddavObject, target_address_book: VdirAddressBook,
+def copy_contact(contact: Contact, target_address_book: VdirAddressBook,
                  delete_source_contact: bool) -> None:
     source_contact_filename = ""
     if delete_source_contact:
@@ -153,16 +153,16 @@ def copy_contact(contact: CarddavObject, target_address_book: VdirAddressBook,
 
 
 def list_address_books(address_books: Union[AddressBookCollection,
-                                            List[VdirAddressBook]]) -> None:
+                                            list[VdirAddressBook]]) -> None:
     table = [["Index", "Address book"]]
     for index, address_book in enumerate(address_books, 1):
         table.append([cast(str, index), address_book.name])
     print(helpers.pretty_print(table))
 
 
-def list_contacts(vcard_list: List[CarddavObject], fields: Iterable[str] = (),
+def list_contacts(vcard_list: list[Contact], fields: Iterable[str] = (),
                   parsable: bool = False) -> None:
-    selected_address_books: List[VdirAddressBook] = []
+    selected_address_books: list[VdirAddressBook] = []
     selected_kinds = set()
     for contact in vcard_list:
         if contact.address_book not in selected_address_books:
@@ -173,7 +173,7 @@ def list_contacts(vcard_list: List[CarddavObject], fields: Iterable[str] = (),
     # default table header
     table_header = ["index", "name", "phone", "email"]
     plural = ""
-    if config.show_kinds or len(selected_kinds) > 1 or CarddavObject._default_kind not in selected_kinds:
+    if config.show_kinds or len(selected_kinds) > 1 or Contact._default_kind not in selected_kinds:
         table_header.append("kind")
     if len(selected_address_books) > 1:
         plural = "s"
@@ -209,22 +209,22 @@ def list_contacts(vcard_list: List[CarddavObject], fields: Iterable[str] = (),
                 row.append(formatter.get_special_field(vcard, field))
             elif field == 'uid':
                 if parsable:
-                    row.append(vcard.uid)
-                elif abook_collection.get_short_uid(vcard.uid):
-                    row.append(abook_collection.get_short_uid(vcard.uid))
+                    row.append(vcard.uid or "")
+                elif uid := abook_collection.get_short_uid(vcard.uid or ""):
+                    row.append(uid)
                 else:
                     row.append("")
             else:
                 row.append(formatter.get_nested_field(vcard, field))
         if parsable:
-            print("\t".join([str(v) for v in row]))
+            print("\t".join(str(v) for v in row))
         else:
             table.append(row)
     if not parsable:
         print(helpers.pretty_print(table))
 
 
-def list_with_headers(the_list: List[str], *headers: str) -> None:
+def list_with_headers(the_list: list[str], *headers: str) -> None:
     table = [list(headers)]
     for row in the_list:
         table.append(row.split("\t"))
@@ -232,14 +232,14 @@ def list_with_headers(the_list: List[str], *headers: str) -> None:
 
 
 def choose_address_book_from_list(header: str, abooks: Union[
-                                  AddressBookCollection, List[VdirAddressBook]]
+                                  AddressBookCollection, list[VdirAddressBook]]
                                   ) -> Optional[VdirAddressBook]:
     """Let the user select one of the given address books
 
     :param header: some text to print in front of the list
     :param abooks: the address books from which to select
     :returns: the selected address book
-    :raises interactive.Canceled: when the user canceled the selection
+    :raises Canceled: when the user canceled the selection
     """
     if not abooks:
         return None
@@ -250,15 +250,15 @@ def choose_address_book_from_list(header: str, abooks: Union[
     return interactive.select(abooks)
 
 
-def choose_vcard_from_list(header: str, vcards: List[CarddavObject],
+def choose_vcard_from_list(header: str, vcards: list[Contact],
                            include_none: bool = False
-                           ) -> Optional[CarddavObject]:
+                           ) -> Optional[Contact]:
     """Let the user select a contact from a list
 
     :param header: some text to print in front of the list
     :param vcards: the contacts from which to select
     :returns: the selected contact
-    :raises interactive.Canceled: when the user canceled the selection
+    :raises Canceled: when the user canceled the selection
     """
     if not vcards:
         return None
@@ -271,22 +271,22 @@ def choose_vcard_from_list(header: str, vcards: List[CarddavObject],
 
 def get_contact_list(address_books: Union[VdirAddressBook,
                                           AddressBookCollection],
-                     query: Query) -> List[CarddavObject]:
+                     query: Query) -> list[Contact]:
     """Find contacts in the given address book grouped, sorted and reversed
     according to the loaded configuration.
 
     :param address_books: the address book to search
     :param query: the query to use when searching
-    :returns: list of found CarddavObject objects
+    :returns: list of found Contact objects
     """
     contacts = address_books.search(query)
     return sort_contacts(contacts, config.reverse, config.group_by_addressbook,
                          config.sort)
 
 
-def sort_contacts(contacts: Iterable[CarddavObject], reverse: bool = False,
-                  group: bool = False, sort: str = "first_name") -> List[
-                      CarddavObject]:
+def sort_contacts(contacts: Iterable[Contact], reverse: bool = False,
+                  group: bool = False, sort: str = "first_name") -> list[
+                      Contact]:
     """Sort a list of contacts
 
     :param contacts: the contact list to sort
@@ -296,7 +296,7 @@ def sort_contacts(contacts: Iterable[CarddavObject], reverse: bool = False,
         "last_name", "formatted_name"
     :returns: sorted contact list
     """
-    keys: List[Callable] = []
+    keys: list[Callable] = []
     if group:
         keys.append(operator.attrgetter("address_book.name"))
     if sort == "first_name":
@@ -309,10 +309,11 @@ def sort_contacts(contacts: Iterable[CarddavObject], reverse: bool = False,
         raise ValueError('sort must be "first_name", "last_name" or '
                          '"formatted_name" not {}.'.format(sort))
     return sorted(contacts, reverse=reverse,
-                  key=lambda x: [unidecode(key(x)).lower() for key in keys])
+                  key=lambda x: [locale.strxfrm(key(x).lower())
+                                 for key in keys])
 
 
-def prepare_search_queries(args: Namespace) -> Dict[str, Query]:
+def prepare_search_queries(args: Namespace) -> dict[str, Query]:
     """Prepare the search query string from the given command line args.
 
     Each address book can get a search query string to filter vCards before
@@ -323,8 +324,8 @@ def prepare_search_queries(args: Namespace) -> Dict[str, Query]:
     :returns: a dict mapping abook names to their loading queries
     """
     # get all possible search queries for address book parsing
-    source_queries: List[Query] = []
-    target_queries: List[Query] = []
+    source_queries: list[Query] = []
+    target_queries: list[Query] = []
     if "source_search_terms" in args:
         source_queries.append(args.source_search_terms)
     if "search_terms" in args:
@@ -338,20 +339,20 @@ def prepare_search_queries(args: Namespace) -> Dict[str, Query]:
     # Get all possible search queries for address book parsing, always
     # depending on the fact if the address book is used to find source or
     # target contacts or both.
-    queries: Dict[str, List[Query]] = {
+    queries: dict[str, list[Query]] = {
         abook.name: [] for abook in config.abooks}
     for name in queries:
         if "addressbook" in args and name in args.addressbook:
             queries[name].append(source_query)
         if "target_addressbook" in args and name in args.target_addressbook:
             queries[name].append(target_query)
-    queries2: Dict[str, Query] = {
+    queries2: dict[str, Query] = {
         n: OrQuery.reduce(q) for n, q in queries.items()}
     logger.debug('Created query: %s', queries)
     return queries2
 
 
-def generate_contact_list(args: Namespace) -> List[CarddavObject]:
+def generate_contact_list(args: Namespace) -> list[Contact]:
     """Find the contact list with which we will work later on
 
     :param args: the command line arguments
@@ -368,7 +369,7 @@ def generate_contact_list(args: Namespace) -> List[CarddavObject]:
 
 
 def new_subcommand(abooks: AddressBookCollection, data: str, open_editor: bool
-                   ) -> None:
+                   ) -> ExitStatus:
     """Create a new contact.
 
     :param abooks: a list of address books that were selected on the command
@@ -376,22 +377,22 @@ def new_subcommand(abooks: AddressBookCollection, data: str, open_editor: bool
     :param data: the data for the new contact as a yaml formatted string
     :param open_editor: whether to open the new contact in the editor after
         creation
-    :raises interactive.Canceled: when the user canceled a selection
+    :raises Canceled: when the user canceled a selection
     """
     # ask for address book, in which to create the new contact
     abook = choose_address_book_from_list(
         "Select address book for new contact", abooks)
     if abook is None:
-        sys.exit("Error: address book list is empty")
+        return "address book list is empty"
     # if there is some data in stdin/the input file
     if data:
         # create new contact from stdin/the input file
         try:
-            new_contact = CarddavObject.from_yaml(
+            new_contact = Contact.from_yaml(
                 abook, data, config.private_objects,
                 config.preferred_vcard_version, config.localize_dates)
         except ValueError as err:
-            sys.exit(str(err))
+            return str(err)
         else:
             new_contact.write_to_file()
         if open_editor:
@@ -411,7 +412,7 @@ def add_email_to_contact(name: str, email_address: str,
     :param email_address: email address of the contact
     :param abooks: the address books that were selected on the command line
     :param skip_already_added: skip if email_address is part of one or more contacts
-    :raises interactive.Canceled: when the user canceled a selection
+    :raises Canceled: when the user canceled a selection
     """
 
     # email address
@@ -552,7 +553,7 @@ def add_email_to_contact(name: str, email_address: str,
 
         # ask for address book, in which to create the new contact
         if not config.abooks:
-            sys.exit("Error: address book list is empty")
+            raise Cancelled("address book list is empty")
         else:
             selected_address_book = choose_address_book_from_list(
                     "Select address book for new contact", config.abooks)
@@ -609,7 +610,7 @@ def add_email_to_contact(name: str, email_address: str,
         if not confirm("Create contact?", False):
             print("Cancelled")
             return
-        selected_vcard = CarddavObject.from_yaml(
+        selected_vcard = Contact.from_yaml(
                 selected_address_book, '\n'.join(template_data),
                 config.private_objects, config.preferred_vcard_version,
                 config.localize_dates)
@@ -639,7 +640,7 @@ def add_email_to_contact(name: str, email_address: str,
         try:
             selected_vcard.add_email(label, email_address)
         except ValueError as err:
-            print(err)
+            print(f"Error: {err}")
         else:
             break
     # save to disk
@@ -647,7 +648,7 @@ def add_email_to_contact(name: str, email_address: str,
     print("Done.\n\n{}".format(selected_vcard.pretty()))
 
 
-def find_email_addresses(text: str, fields: List[str]) -> List[Address]:
+def find_email_addresses(text: str, fields: list[str]) -> list[Address]:
     """Search the text for email addresses in the given fields.
 
     :param text: the text to search for email addresses
@@ -656,7 +657,7 @@ def find_email_addresses(text: str, fields: List[str]) -> List[Address]:
     """
     message = message_from_string(text, policy=SMTP_POLICY)
 
-    def extract_addresses(header) -> List[Address]:
+    def extract_addresses(header) -> list[Address]:
         if header and isinstance(header, (AddressHeader, Group)):
             return list(header.addresses)
         return []
@@ -677,19 +678,19 @@ def find_email_addresses(text: str, fields: List[str]) -> List[Address]:
 def add_email_subcommand(
         text: str,
         abooks: AddressBookCollection,
-        fields: List[str],
-        skip_already_added: bool) -> None:
+        fields: list[str],
+        skip_already_added: bool) -> ExitStatus:
     """Add a new email address to contacts, creating new contacts if necessary.
 
     :param text: the input text to search for the new email
     :param abooks: the address books that were selected on the command line
     :param field: the header field to extract contacts from
     :param skip_already_added: skip already known email addresses
-    :raises interactive.Canceled: when the user canceled a selection
+    :raises Canceled: when the user canceled a selection
     """
     email_addresses = find_email_addresses(text, fields)
     if not email_addresses:
-        sys.exit("No email addresses found in fields {}".format(fields))
+        return f"No email addresses found in fields {fields}"
 
     print("Khard: Add email addresses to contacts")
 
@@ -704,8 +705,8 @@ def add_email_subcommand(
     print("No more email addresses")
 
 
-def birthdays_subcommand(vcard_list: List[CarddavObject], parsable: bool
-                         ) -> None:
+def birthdays_subcommand(vcard_list: list[Contact], parsable: bool
+                         ) -> ExitStatus:
     """Print birthday contact table.
 
     :param vcard_list: the vCards to search for matching entries which should
@@ -721,7 +722,7 @@ def birthdays_subcommand(vcard_list: List[CarddavObject], parsable: bool
                     if isinstance(x.birthday, datetime.datetime)
                     else (0, 0, x.birthday))
     # add to string list
-    birthday_list: List[str] = []
+    birthday_list: list[str] = []
     formatter = Formatter(config.display, config.preferred_email_address_type,
                           config.preferred_phone_number_type,
                           config.show_nicknames, parsable)
@@ -747,11 +748,11 @@ def birthdays_subcommand(vcard_list: List[CarddavObject], parsable: bool
     else:
         if not parsable:
             print("Found no birthdays")
-        sys.exit(1)
+        return 1
 
 
-def phone_subcommand(search_terms: Query, vcard_list: List[CarddavObject],
-        parsable: bool) -> None:
+def phone_subcommand(search_terms: Query, vcard_list: list[Contact],
+        parsable: bool) -> ExitStatus:
     """Print a phone application friendly contact table.
 
     :param search_terms: used as search term to filter the contacts before
@@ -763,7 +764,7 @@ def phone_subcommand(search_terms: Query, vcard_list: List[CarddavObject],
     formatter = Formatter(config.display, config.preferred_email_address_type,
                           config.preferred_phone_number_type,
                           config.show_nicknames, parsable)
-    numbers: List[str] = []
+    numbers: list[str] = []
     for vcard in vcard_list:
         field_line_list = []
         for type, number_list in sorted(vcard.phone_numbers.items(),
@@ -787,12 +788,12 @@ def phone_subcommand(search_terms: Query, vcard_list: List[CarddavObject],
     else:
         if not parsable:
             print("Found no phone numbers")
-        sys.exit(1)
+        return 1
 
 
 def post_address_subcommand(search_terms: Query,
-        vcard_list: List[CarddavObject], parsable: bool
-                            ) -> None:
+        vcard_list: list[Contact], parsable: bool
+                            ) -> ExitStatus:
     """Print a contact table with all postal / mailing addresses
 
     :param search_terms: used as search term to filter the contacts before
@@ -804,7 +805,7 @@ def post_address_subcommand(search_terms: Query,
     formatter = Formatter(config.display, config.preferred_email_address_type,
                           config.preferred_phone_number_type,
                           config.show_nicknames, parsable)
-    addresses: List[str] = []
+    addresses: list[str] = []
     for vcard in vcard_list:
         name = formatter.get_special_field(vcard, "name")
         # create post address line list
@@ -832,11 +833,11 @@ def post_address_subcommand(search_terms: Query,
     else:
         if not parsable:
             print("Found no post addresses")
-        sys.exit(1)
+        return 1
 
 
-def email_subcommand(search_terms: Query, vcard_list: List[CarddavObject],
-                     parsable: bool, remove_first_line: bool) -> None:
+def email_subcommand(search_terms: Query, vcard_list: list[Contact],
+                     parsable: bool, remove_first_line: bool) -> ExitStatus:
     """Print a mail client friendly contacts table that is compatible with the
     default format used by mutt.
     Output format:
@@ -858,7 +859,7 @@ def email_subcommand(search_terms: Query, vcard_list: List[CarddavObject],
     formatter = Formatter(config.display, config.preferred_email_address_type,
                           config.preferred_phone_number_type,
                           config.show_nicknames, parsable)
-    emails: List[str] = []
+    emails: list[str] = []
     for vcard in vcard_list:
         field_line_list = []
         for type, email_list in sorted(vcard.emails.items(),
@@ -887,11 +888,11 @@ def email_subcommand(search_terms: Query, vcard_list: List[CarddavObject],
             print("Found no email addresses")
         elif not remove_first_line:
             print("searching for '{}' ...".format(search_terms))
-        sys.exit(1)
+        return 1
 
 
 def _filter_email_post_or_phone_number_results(search_terms: Query,
-        field_line_list: List[str]) -> List[str]:
+        field_line_list: list[str]) -> list[str]:
     """Filter the created output of phone_subcommand, post_address_subcommand
     and email_subcommand by the given search term again.
     If no match is found, return the complete input list
@@ -907,8 +908,8 @@ def _filter_email_post_or_phone_number_results(search_terms: Query,
     return matched_line_list if matched_line_list else field_line_list
 
 
-def list_subcommand(vcard_list: List[CarddavObject], parsable: bool,
-                    fields: List[str]) -> None:
+def list_subcommand(vcard_list: list[Contact], parsable: bool,
+                    fields: list[str]) -> ExitStatus:
     """Print a user friendly contacts table.
 
     :param vcard_list: the vCards to print
@@ -918,14 +919,14 @@ def list_subcommand(vcard_list: List[CarddavObject], parsable: bool,
     if not vcard_list:
         if not parsable:
             print("Found no contacts")
-        sys.exit(1)
+        return 1
     else:
         list_contacts(vcard_list, fields, parsable)
 
 
-def modify_subcommand(selected_vcard: CarddavObject,
+def modify_subcommand(selected_vcard: Contact,
                       input_from_stdin_or_file: str, open_editor: bool,
-                      source: bool = False) -> None:
+                      source: bool = False) -> ExitStatus:
     """Modify a contact in an external editor.
 
     :param selected_vcard: the contact to modify
@@ -939,19 +940,19 @@ def modify_subcommand(selected_vcard: CarddavObject,
     if source:
         editor = interactive.Editor(config.editor, config.merge_editor)
         editor.edit_files(selected_vcard.filename)
-        return
+        return None
     # show warning, if vCard version of selected contact is not 3.0 or 4.0
     if not version_check(selected_vcard, "selected contact"):
-        return
+        return None
     # if there is some data in stdin
     if input_from_stdin_or_file:
         # create new contact from stdin
         try:
-            new_contact = CarddavObject.clone_with_yaml_update(
+            new_contact = Contact.clone_with_yaml_update(
                 selected_vcard, input_from_stdin_or_file,
                 config.localize_dates)
         except ValueError as err:
-            sys.exit(str(err))
+            return str(err)
         if selected_vcard == new_contact:
             print("Nothing changed\n\n{}".format(new_contact.pretty()))
         else:
@@ -968,7 +969,7 @@ def modify_subcommand(selected_vcard: CarddavObject,
         modify_existing_contact(selected_vcard)
 
 
-def remove_subcommand(selected_vcard: CarddavObject, force: bool) -> None:
+def remove_subcommand(selected_vcard: Contact, force: bool) -> None:
     """Remove a contact from the address book.
 
     :param selected_vcard: the contact to delete
@@ -984,15 +985,15 @@ def remove_subcommand(selected_vcard: CarddavObject, force: bool) -> None:
         selected_vcard.formatted_name))
 
 
-def merge_subcommand(vcards: List[CarddavObject],
+def merge_subcommand(vcards: list[Contact],
                      abooks: AddressBookCollection, search_terms: Query
-                     ) -> None:
+                     ) -> ExitStatus:
     """Merge two contacts into one.
 
     :param vcards: the vCards from which to choose contacts for merging
     :param abooks: the address books to use to find the target contact
     :param search_terms: the search terms to find the target contact
-    :raises interactive.Canceled: when the user canceled a selection
+    :raises Canceled: when the user canceled a selection
     """
     # Find possible target contacts.
     target_vcards = get_contact_list(abooks, search_terms)
@@ -1000,7 +1001,7 @@ def merge_subcommand(vcards: List[CarddavObject],
     source_vcard = choose_vcard_from_list("Select contact from which to merge",
                                           vcards)
     if source_vcard is None:
-        sys.exit("Found no source contact for merging")
+        return "Found no source contact for merging"
     else:
         print("Merge from {} from address book {}\n\n".format(
             source_vcard, source_vcard.address_book))
@@ -1008,7 +1009,7 @@ def merge_subcommand(vcards: List[CarddavObject],
     target_vcard = choose_vcard_from_list("Select contact into which to merge",
                                           target_vcards)
     if target_vcard is None:
-        sys.exit("Found no target contact for merging")
+        return "Found no target contact for merging"
     else:
         print("Merge into {} from address book {}\n\n".format(
             target_vcard, target_vcard.address_book))
@@ -1019,21 +1020,21 @@ def merge_subcommand(vcards: List[CarddavObject],
         merge_existing_contacts(source_vcard, target_vcard, True)
 
 
-def copy_or_move_subcommand(action: str, vcards: List[CarddavObject],
+def copy_or_move_subcommand(action: str, vcards: list[Contact],
                             target_address_books: AddressBookCollection
-                            ) -> None:
+                            ) -> ExitStatus:
     """Copy or move a contact to a different address book.
 
     :param action: the string "copy" or "move" to indicate what to do
     :param vcards: the contact list from which to select one for the action
     :param target_address_books: the target address books
-    :raises interactive.Canceled: when the user canceled a selection
+    :raises Canceled: when the user canceled a selection
     """
     # get the source vCard, which to copy or move
     source_vcard = choose_vcard_from_list(
         "Select contact to {}".format(action.title()), vcards)
     if source_vcard is None:
-        sys.exit("Found no contact")
+        return "Found no contact"
     else:
         print("{} contact {} from address book {}".format(
             action.title(), source_vcard, source_vcard.address_book))
@@ -1041,15 +1042,14 @@ def copy_or_move_subcommand(action: str, vcards: List[CarddavObject],
     # get target address book
     if len(target_address_books) == 1 \
             and target_address_books[0] == source_vcard.address_book:
-        sys.exit("The address book {} already contains the contact {}".format(
-            target_address_books[0], source_vcard))
+        return f"The address book {target_address_books[0]} already contains the contact {source_vcard}"
     else:
         available_address_books = [abook for abook in target_address_books
                                    if abook != source_vcard.address_book]
         target_abook = choose_address_book_from_list(
             "Select target address book", available_address_books)
         if target_abook is None:
-            sys.exit("Error: address book list is empty")
+            return "address book list is empty"
 
     # check if a contact already exists in the target address book
     target_vcard = choose_vcard_from_list(
@@ -1094,7 +1094,8 @@ def copy_or_move_subcommand(action: str, vcards: List[CarddavObject],
                 break
 
 
-def main(argv: List[str] = sys.argv[1:]) -> None:
+def main(argv: list[str] = sys.argv[1:]) -> ExitStatus:
+    locale.setlocale(locale.LC_ALL, "")
     args, conf = cli.init(argv)
 
     # store the config instance in the module level variable
@@ -1105,7 +1106,7 @@ def main(argv: List[str] = sys.argv[1:]) -> None:
     # options and can directly be run.
     if args.action == "addressbooks":
         print('\n'.join(str(book) for book in config.abooks))
-        return
+        return None
     if args.action == "template":
         print("# Contact template for khard version {}\n#\n"
               "# Use this yaml formatted template to create a new contact:\n"
@@ -1113,7 +1114,7 @@ def main(argv: List[str] = sys.argv[1:]) -> None:
               "#   or with: cat template.yaml | khard new -a address_book\n"
               "\n{}".format(khard_version, helpers.get_new_contact_template(
                   config.private_objects)))
-        return
+        return None
 
     search_queries = prepare_search_queries(args)
 
@@ -1126,16 +1127,15 @@ def main(argv: List[str] = sys.argv[1:]) -> None:
             args.target_addressbook = config.get_address_books(
                 args.target_addressbook, search_queries)
     except AddressBookParseError as err:
-        sys.exit("{}\nUse --debug for more information or --skip-unparsable "
-                 "to proceed".format(err))
+        return f"{err}\nUse --debug for more information or --skip-unparsable to proceed"
     except AddressBookNameError as err:
-        sys.exit(str(err))
+        return str(err)
 
     vcard_list = generate_contact_list(args)
 
     if args.action == "filename":
         print('\n'.join(contact.filename for contact in vcard_list))
-        return
+        return None
 
     # read from template file or stdin if available
     input_from_stdin_or_file = ""
@@ -1146,14 +1146,13 @@ def main(argv: List[str] = sys.argv[1:]) -> None:
                 with open(args.input_file, "r") as infile:
                     input_from_stdin_or_file = infile.read()
             except OSError as err:
-                sys.exit("Error: {}\n       File: {}".format(err.strerror,
-                                                             err.filename))
+                return "Error: {}\n       File: {}".format(err.strerror, err.filename)
         elif not sys.stdin.isatty():
             # try to read from stdin
             try:
                 input_from_stdin_or_file = sys.stdin.read()
             except OSError:
-                sys.exit("Error: Can't read from stdin")
+                return "Error: Can't read from stdin"
             # try to reopen console
             # otherwise further user interaction is not possible (for example
             # selecting a contact from the contact table)
@@ -1164,25 +1163,25 @@ def main(argv: List[str] = sys.argv[1:]) -> None:
 
     # these listing commands do not require any user interaction
     if args.action == "birthdays":
-        birthdays_subcommand(vcard_list, args.parsable)
+        return birthdays_subcommand(vcard_list, args.parsable)
     elif args.action == "phone":
-        phone_subcommand(args.search_terms, vcard_list, args.parsable)
+        return phone_subcommand(args.search_terms, vcard_list, args.parsable)
     elif args.action == "postaddress":
-        post_address_subcommand(args.search_terms, vcard_list, args.parsable)
+        return post_address_subcommand(args.search_terms, vcard_list, args.parsable)
     elif args.action == "email":
-        email_subcommand(args.search_terms, vcard_list,
-                         args.parsable, args.remove_first_line)
+        return email_subcommand(args.search_terms, vcard_list, args.parsable,
+                                args.remove_first_line)
     elif args.action == "list":
-        list_subcommand(vcard_list, args.parsable, args.fields)
+        return list_subcommand(vcard_list, args.parsable, args.fields)
 
     else:
         # these commands require user interaction
         try:
             if args.action == "new":
-                new_subcommand(args.addressbook, input_from_stdin_or_file,
-                               args.open_editor)
+                return new_subcommand(args.addressbook, input_from_stdin_or_file,
+                                      args.open_editor)
             elif args.action == "add-email":
-                add_email_subcommand(input_from_stdin_or_file,
+                return add_email_subcommand(input_from_stdin_or_file,
                                      args.addressbook, args.headers,
                                      args.skip_already_added)
             elif args.action in ["show", "edit", "remove"]:
@@ -1190,7 +1189,7 @@ def main(argv: List[str] = sys.argv[1:]) -> None:
                     "Select contact for {} action".format(args.action.title()),
                     vcard_list)
                 if selected_vcard is None:
-                    sys.exit("Found no contact")
+                    return "Found no contact"
                 if args.action == "show":
                     if args.format == "pretty":
                         output = selected_vcard.pretty()
@@ -1204,15 +1203,19 @@ def main(argv: List[str] = sys.argv[1:]) -> None:
                                      selected_vcard.to_yaml())
                     args.output_file.write(output)
                 elif args.action == "edit":
-                    modify_subcommand(selected_vcard, input_from_stdin_or_file,
+                    return modify_subcommand(selected_vcard, input_from_stdin_or_file,
                                       args.open_editor, args.format == 'vcard')
                 elif args.action == "remove":
                     remove_subcommand(selected_vcard, args.force)
             elif args.action == "merge":
-                merge_subcommand(vcard_list, args.target_addressbook,
-                                 args.target_contact)
+                return merge_subcommand(vcard_list, args.target_addressbook,
+                                        args.target_contact)
             elif args.action in ["copy", "move"]:
-                copy_or_move_subcommand(
-                    args.action, vcard_list, args.target_addressbook)
-        except interactive.Canceled as ex:
-            sys.exit(str(ex))
+                return copy_or_move_subcommand(args.action, vcard_list,
+                                               args.target_addressbook)
+        except Cancelled as ex:
+            if ex.code == 0:
+                print(ex)
+            else:
+                print(f"Error: {ex}")
+            return ex.code
